@@ -13,9 +13,17 @@ module Hledger.Cli (
                      module Hledger.Cli.Stats,
                      module Hledger.Cli.Options,
                      module Hledger.Cli.Utils,
+                     module Hledger.Cli.Version,
                      tests_Hledger_Cli
               )
 where
+import Control.Monad
+import qualified Data.Map as Map
+import Data.Time.Calendar
+import System.Time (ClockTime(TOD))
+import Test.HUnit
+
+import Hledger
 import Hledger.Cli.Add
 import Hledger.Cli.Balance
 import Hledger.Cli.Convert
@@ -25,19 +33,11 @@ import Hledger.Cli.Register
 import Hledger.Cli.Stats
 import Hledger.Cli.Options
 import Hledger.Cli.Utils
-
-
-import qualified Data.Map as Map
-import System.Time (ClockTime(TOD))
-
-import Hledger.Data  -- including testing utils in Hledger.Data.Utils
-import Hledger.Read
-import Hledger.Read.JournalReader (someamount)
-
+import Hledger.Cli.Version
 
 -- | hledger and hledger-lib's unit tests aggregated from all modules
 -- plus some more which are easier to define here for now.
--- tests_Hledger_Cli1 :: Test
+tests_Hledger_Cli :: Test
 tests_Hledger_Cli = TestList
  [
     tests_Hledger_Data
@@ -83,8 +83,20 @@ tests_Hledger_Cli = TestList
                             "2008/12/07 Four\n  outer:why  $-4\n  outer:zed  $4\n" ++
                             "2008/12/07 Five\n  foo  $-5\n  bar  $5\n"
                            )
+
+   ,"account directive should preserve \"virtual\" posting type" ~: do
+      j <- readJournal Nothing "!account test\n2008/12/07 One\n  (from)  $-1\n  (to)  $1\n" >>= either error' return
+      let p = head $ tpostings $ head $ jtxns j
+      assertBool "" $ (paccount p) == "test:from"
+      assertBool "" $ (ptype p) == VirtualPosting
+
    ]
 
+   ,"account aliases" ~: do
+      Right j <- readJournal Nothing "!alias expenses = equity:draw:personal\n1/1\n (expenses:food)  1\n"
+      let p = head $ tpostings $ head $ jtxns j
+      assertBool "" $ paccount p == "equity:draw:personal:food"
+
   ,"ledgerAccountNames" ~:
     ledgerAccountNames ledger7 `is`
      ["assets","assets:cash","assets:checking","assets:saving","equity","equity:opening balances",
@@ -92,15 +104,14 @@ tests_Hledger_Cli = TestList
       "liabilities","liabilities:credit cards","liabilities:credit cards:discover"]
 
   ,"balance report tests" ~:
-   let (opts,args) `gives` es = do 
-        l <- samplejournalwithopts opts args
-        t <- getCurrentLocalTime
-        balanceReportAsText opts (balanceReport opts (optsToFilterSpec opts args t) l) `is` unlines es
+   let opts `gives` es = do
+        j <- samplejournal
+        d <- getCurrentDay
+        accountsReportAsText opts (accountsReport opts (optsToFilterSpec opts d) j) `is` es
    in TestList
    [
-
     "balance report with no args" ~:
-    ([], []) `gives`
+    defreportopts `gives`
     ["                 $-1  assets"
     ,"                  $1    bank:saving"
     ,"                 $-2    cash"
@@ -116,7 +127,7 @@ tests_Hledger_Cli = TestList
     ]
 
    ,"balance report can be limited with --depth" ~:
-    ([Depth "1"], []) `gives`
+    defreportopts{depth_=Just 1} `gives`
     ["                 $-1  assets"
     ,"                  $2  expenses"
     ,"                 $-2  income"
@@ -126,7 +137,7 @@ tests_Hledger_Cli = TestList
     ]
     
    ,"balance report with account pattern o" ~:
-    ([SubTotal], ["o"]) `gives`
+    defreportopts{patterns_=["o"]} `gives`
     ["                  $1  expenses:food"
     ,"                 $-2  income"
     ,"                 $-1    gifts"
@@ -136,7 +147,7 @@ tests_Hledger_Cli = TestList
     ]
 
    ,"balance report with account pattern o and --depth 1" ~:
-    ([Depth "1"], ["o"]) `gives`
+    defreportopts{patterns_=["o"],depth_=Just 1} `gives`
     ["                  $1  expenses"
     ,"                 $-2  income"
     ,"--------------------"
@@ -144,7 +155,7 @@ tests_Hledger_Cli = TestList
     ]
 
    ,"balance report with account pattern a" ~:
-    ([], ["a"]) `gives`
+    defreportopts{patterns_=["a"]} `gives`
     ["                 $-1  assets"
     ,"                  $1    bank:saving"
     ,"                 $-2    cash"
@@ -155,7 +166,7 @@ tests_Hledger_Cli = TestList
     ]
 
    ,"balance report with account pattern e" ~:
-    ([], ["e"]) `gives`
+    defreportopts{patterns_=["e"]} `gives`
     ["                 $-1  assets"
     ,"                  $1    bank:saving"
     ,"                 $-2    cash"
@@ -171,7 +182,7 @@ tests_Hledger_Cli = TestList
     ]
 
    ,"balance report with unmatched parent of two matched subaccounts" ~: 
-    ([], ["cash","saving"]) `gives`
+    defreportopts{patterns_=["cash","saving"]} `gives`
     ["                 $-1  assets"
     ,"                  $1    bank:saving"
     ,"                 $-2    cash"
@@ -180,14 +191,14 @@ tests_Hledger_Cli = TestList
     ]
 
    ,"balance report with multi-part account name" ~: 
-    ([], ["expenses:food"]) `gives`
+    defreportopts{patterns_=["expenses:food"]} `gives`
     ["                  $1  expenses:food"
     ,"--------------------"
     ,"                  $1"
     ]
 
    ,"balance report with negative account pattern" ~:
-    ([], ["not:assets"]) `gives`
+    defreportopts{patterns_=["not:assets"]} `gives`
     ["                  $2  expenses"
     ,"                  $1    food"
     ,"                  $1    supplies"
@@ -200,20 +211,20 @@ tests_Hledger_Cli = TestList
     ]
 
    ,"balance report negative account pattern always matches full name" ~: 
-    ([], ["not:e"]) `gives`
+    defreportopts{patterns_=["not:e"]} `gives`
     ["--------------------"
     ,"                   0"
     ]
 
    ,"balance report negative patterns affect totals" ~: 
-    ([], ["expenses","not:food"]) `gives`
+    defreportopts{patterns_=["expenses","not:food"]} `gives`
     ["                  $1  expenses:supplies"
     ,"--------------------"
     ,"                  $1"
     ]
 
    ,"balance report with -E shows zero-balance accounts" ~:
-    ([SubTotal,Empty], ["assets"]) `gives`
+    defreportopts{patterns_=["assets"],empty_=True} `gives`
     ["                 $-1  assets"
     ,"                  $1    bank"
     ,"                   0      checking"
@@ -231,8 +242,7 @@ tests_Hledger_Cli = TestList
              ,"  c:d                   "
              ]) >>= either error' return
       let j' = journalCanonicaliseAmounts $ journalConvertAmountsToCost j -- enable cost basis adjustment
-      balanceReportAsText [] (balanceReport [] nullfilterspec j') `is`
-       unlines
+      accountsReportAsText defreportopts (accountsReport defreportopts nullfilterspec j') `is`
         ["                $500  a:b"
         ,"               $-500  c:d"
         ,"--------------------"
@@ -240,14 +250,13 @@ tests_Hledger_Cli = TestList
         ]
 
    ,"balance report elides zero-balance root account(s)" ~: do
-      l <- readJournalWithOpts []
+      j <- readJournal'
              (unlines
               ["2008/1/1 one"
               ,"  test:a  1"
               ,"  test:b"
               ])
-      balanceReportAsText [] (balanceReport [] nullfilterspec l) `is`
-       unlines
+      accountsReportAsText defreportopts (accountsReport defreportopts nullfilterspec j) `is`
         ["                   1  test:a"
         ,"                  -1  test:b"
         ,"--------------------"
@@ -271,8 +280,8 @@ tests_Hledger_Cli = TestList
   --     `is` "aa:aa:aaaaaaaaaaaaaa")
 
   ,"default year" ~: do
-    rl <- readJournal Nothing defaultyear_journal_str >>= either error' return
-    tdate (head $ jtxns rl) `is` fromGregorian 2009 1 1
+    j <- readJournal Nothing defaultyear_journal_str >>= either error' return
+    tdate (head $ jtxns j) `is` fromGregorian 2009 1 1
     return ()
 
   ,"print report tests" ~: TestList
@@ -280,10 +289,10 @@ tests_Hledger_Cli = TestList
 
    "print expenses" ~:
    do 
-    let args = ["expenses"]
-    l <- samplejournalwithopts [] args
-    t <- getCurrentLocalTime
-    showTransactions (optsToFilterSpec [] args t) l `is` unlines
+    let opts = defreportopts{patterns_=["expenses"]}
+    j <- samplejournal
+    d <- getCurrentDay
+    showTransactions opts (optsToFilterSpec opts d) j `is` unlines
      ["2008/06/03 * eat & shop"
      ,"    expenses:food                $1"
      ,"    expenses:supplies            $1"
@@ -293,9 +302,10 @@ tests_Hledger_Cli = TestList
 
   , "print report with depth arg" ~:
    do 
-    l <- samplejournal
-    t <- getCurrentLocalTime
-    showTransactions (optsToFilterSpec [Depth "2"] [] t) l `is` unlines
+    let opts = defreportopts{depth_=Just 2}
+    j <- samplejournal
+    d <- getCurrentDay
+    showTransactions opts (optsToFilterSpec opts d) j `is` unlines
       ["2008/01/01 income"
       ,"    income:salary           $-1"
       ,""
@@ -322,8 +332,9 @@ tests_Hledger_Cli = TestList
 
    "register report with no args" ~:
    do 
-    l <- samplejournal
-    (registerReportAsText [] $ registerReport [] (optsToFilterSpec [] [] t1) l) `is` unlines
+    j <- samplejournal
+    let opts = defreportopts
+    (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` unlines
      ["2008/01/01 income               assets:bank:checking             $1           $1"
      ,"                                income:salary                   $-1            0"
      ,"2008/06/01 gift                 assets:bank:checking             $1           $1"
@@ -339,9 +350,9 @@ tests_Hledger_Cli = TestList
 
   ,"register report with cleared option" ~:
    do 
-    let opts = [Cleared]
-    l <- readJournalWithOpts opts sample_journal_str
-    (registerReportAsText opts $ registerReport opts (optsToFilterSpec opts [] t1) l) `is` unlines
+    let opts = defreportopts{cleared_=True}
+    j <- readJournal' sample_journal_str
+    (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` unlines
      ["2008/06/03 eat & shop           expenses:food                    $1           $1"
      ,"                                expenses:supplies                $1           $2"
      ,"                                assets:cash                     $-2            0"
@@ -351,9 +362,9 @@ tests_Hledger_Cli = TestList
 
   ,"register report with uncleared option" ~:
    do 
-    let opts = [UnCleared]
-    l <- readJournalWithOpts opts sample_journal_str
-    (registerReportAsText opts $ registerReport opts (optsToFilterSpec opts [] t1) l) `is` unlines
+    let opts = defreportopts{uncleared_=True}
+    j <- readJournal' sample_journal_str
+    (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` unlines
      ["2008/01/01 income               assets:bank:checking             $1           $1"
      ,"                                income:salary                   $-1            0"
      ,"2008/06/01 gift                 assets:bank:checking             $1           $1"
@@ -364,7 +375,7 @@ tests_Hledger_Cli = TestList
 
   ,"register report sorts by date" ~:
    do 
-    l <- readJournalWithOpts [] $ unlines
+    j <- readJournal' $ unlines
         ["2008/02/02 a"
         ,"  b  1"
         ,"  c"
@@ -373,28 +384,31 @@ tests_Hledger_Cli = TestList
         ,"  e  1"
         ,"  f"
         ]
-    registerdates (registerReportAsText [] $ registerReport [] (optsToFilterSpec [] [] t1) l) `is` ["2008/01/01","2008/02/02"]
+    let opts = defreportopts
+    registerdates (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` ["2008/01/01","2008/02/02"]
 
   ,"register report with account pattern" ~:
    do
-    l <- samplejournal
-    (registerReportAsText [] $ registerReport [] (optsToFilterSpec [] ["cash"] t1) l) `is` unlines
+    j <- samplejournal
+    let opts = defreportopts{patterns_=["cash"]}
+    (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` unlines
      ["2008/06/03 eat & shop           assets:cash                     $-2          $-2"
      ]
 
   ,"register report with account pattern, case insensitive" ~:
    do 
-    l <- samplejournal
-    (registerReportAsText [] $ registerReport [] (optsToFilterSpec [] ["cAsH"] t1) l) `is` unlines
+    j <- samplejournal
+    let opts = defreportopts{patterns_=["cAsH"]}
+    (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` unlines
      ["2008/06/03 eat & shop           assets:cash                     $-2          $-2"
      ]
 
   ,"register report with display expression" ~:
    do 
-    l <- samplejournal
+    j <- samplejournal
     let gives displayexpr = 
-            (registerdates (registerReportAsText opts $ registerReport opts (optsToFilterSpec opts [] t1) l) `is`)
-                where opts = [Display displayexpr]
+            (registerdates (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is`)
+                where opts = defreportopts{display_=Just displayexpr}
     "d<[2008/6/2]"  `gives` ["2008/01/01","2008/06/01"]
     "d<=[2008/6/2]" `gives` ["2008/01/01","2008/06/01","2008/06/02"]
     "d=[2008/6/2]"  `gives` ["2008/06/02"]
@@ -403,19 +417,19 @@ tests_Hledger_Cli = TestList
 
   ,"register report with period expression" ~:
    do 
-    l <- samplejournal
+    j <- samplejournal
     let periodexpr `gives` dates = do
-          l' <- samplejournalwithopts opts []
-          registerdates (registerReportAsText opts $ registerReport opts (optsToFilterSpec opts [] t1) l') `is` dates
-              where opts = [Period periodexpr]
+          j' <- samplejournal
+          registerdates (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j') `is` dates
+              where opts = defreportopts{period_=maybePeriod date1 periodexpr}
     ""     `gives` ["2008/01/01","2008/06/01","2008/06/02","2008/06/03","2008/12/31"]
     "2008" `gives` ["2008/01/01","2008/06/01","2008/06/02","2008/06/03","2008/12/31"]
     "2007" `gives` []
     "june" `gives` ["2008/06/01","2008/06/02","2008/06/03"]
     "monthly" `gives` ["2008/01/01","2008/06/01","2008/12/01"]
     "quarterly" `gives` ["2008/01/01","2008/04/01","2008/10/01"]
-    let opts = [Period "yearly"]
-    (registerReportAsText opts $ registerReport opts (optsToFilterSpec opts [] t1) l) `is` unlines
+    let opts = defreportopts{period_=maybePeriod date1 "yearly"}
+    (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` unlines
      ["2008/01/01 - 2008/12/31         assets:bank:saving               $1           $1"
      ,"                                assets:cash                     $-2          $-1"
      ,"                                expenses:food                    $1            0"
@@ -424,18 +438,18 @@ tests_Hledger_Cli = TestList
      ,"                                income:salary                   $-1          $-1"
      ,"                                liabilities:debts                $1            0"
      ]
-    let opts = [Period "quarterly"]
-    registerdates (registerReportAsText opts $ registerReport opts (optsToFilterSpec opts [] t1) l) `is` ["2008/01/01","2008/04/01","2008/10/01"]
-    let opts = [Period "quarterly",Empty]
-    registerdates (registerReportAsText opts $ registerReport opts (optsToFilterSpec opts [] t1) l) `is` ["2008/01/01","2008/04/01","2008/07/01","2008/10/01"]
+    let opts = defreportopts{period_=maybePeriod date1 "quarterly"}
+    registerdates (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` ["2008/01/01","2008/04/01","2008/10/01"]
+    let opts = defreportopts{period_=maybePeriod date1 "quarterly",empty_=True}
+    registerdates (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` ["2008/01/01","2008/04/01","2008/07/01","2008/10/01"]
 
   ]
 
   , "register report with depth arg" ~:
    do 
-    l <- samplejournal
-    let opts = [Depth "2"]
-    (registerReportAsText opts $ registerReport opts (optsToFilterSpec opts [] t1) l) `is` unlines
+    j <- samplejournal
+    let opts = defreportopts{depth_=Just 2}
+    (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` unlines
      ["2008/01/01 income               assets:bank                      $1           $1"
      ,"                                income:salary                   $-1            0"
      ,"2008/06/01 gift                 assets:bank                      $1           $1"
@@ -454,9 +468,10 @@ tests_Hledger_Cli = TestList
   ,"show hours" ~: show (hours 1) ~?= "1.0h"
 
   ,"unicode in balance layout" ~: do
-    l <- readJournalWithOpts []
+    j <- readJournal'
       "2009/01/01 * медвежья шкура\n  расходы:покупки  100\n  актив:наличные\n"
-    balanceReportAsText [] (balanceReport [] (optsToFilterSpec [] [] t1) l) `is` unlines
+    let opts = defreportopts
+    accountsReportAsText opts (accountsReport opts (optsToFilterSpec opts date1) j) `is`
       ["                -100  актив:наличные"
       ,"                 100  расходы:покупки"
       ,"--------------------"
@@ -464,9 +479,10 @@ tests_Hledger_Cli = TestList
       ]
 
   ,"unicode in register layout" ~: do
-    l <- readJournalWithOpts []
+    j <- readJournal'
       "2009/01/01 * медвежья шкура\n  расходы:покупки  100\n  актив:наличные\n"
-    (registerReportAsText [] $ registerReport [] (optsToFilterSpec [] [] t1) l) `is` unlines
+    let opts = defreportopts
+    (postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts date1) j) `is` unlines
       ["2009/01/01 медвежья шкура       расходы:покупки                 100          100"
       ,"                                актив:наличные                 -100            0"]
 
@@ -480,10 +496,10 @@ tests_Hledger_Cli = TestList
   
 -- fixtures/test data
 
-t1 = LocalTime date1 midday where date1 = parsedate "2008/11/26"
+date1 = parsedate "2008/11/26"
+-- t1 = LocalTime date1 midday
 
-samplejournal = readJournalWithOpts [] sample_journal_str
-samplejournalwithopts opts _ = readJournalWithOpts opts sample_journal_str
+samplejournal = readJournal' sample_journal_str
 
 sample_journal_str = unlines
  ["; A sample journal file."
@@ -906,4 +922,3 @@ journalWithAmounts as =
         []
         (TOD 0 0)
     where parse = fromparse . parseWithCtx nullctx someamount
-
