@@ -1,35 +1,39 @@
-{-# LANGUAGE CPP #-}
 {-| 
 
 A history-aware add command to help with data entry.
 
 Note: this might not be sensible, but add has some aspirations of being
-both user-fiendly and pipeable/scriptable and for this reason
+both user-friendly and pipeable/scriptable and for this reason
 informational messages are mostly written to stderr rather than stdout.
 
 -}
 
 module Hledger.Cli.Add
 where
-import Hledger.Data
-import Hledger.Read.JournalReader (someamount)
-import Hledger.Cli.Options
-import Hledger.Cli.Register (registerReport, registerReportAsText)
-import Prelude hiding (putStr, putStrLn, appendFile)
-import Hledger.Data.UTF8 (putStr, putStrLn, appendFile)
-
+import Control.Exception (throw)
+import Control.Monad
+import Control.Monad.Trans (liftIO)
+import Data.Char (toUpper)
+import Data.List
+import Data.Maybe
+import Data.Time.Calendar
+import Safe (headMay)
+import System.Console.Haskeline (InputT, runInputT, defaultSettings, setComplete, getInputLine)
+import System.Console.Haskeline.Completion
 import System.IO ( stderr, hPutStrLn, hPutStr )
 import System.IO.Error
 import Text.ParserCombinators.Parsec
-import Hledger.Cli.Utils (readJournalWithOpts)
+import Text.Printf
 import qualified Data.Foldable as Foldable (find)
-import System.Console.Haskeline (
-            InputT, runInputT, defaultSettings, setComplete, getInputLine)
-import Control.Monad.Trans (liftIO)
-import System.Console.Haskeline.Completion
 import qualified Data.Set as Set
-import Safe (headMay)
-import Control.Exception (throw)
+
+import Hledger
+import Prelude hiding (putStr, putStrLn, appendFile)
+import Hledger.Utils.UTF8 (putStr, putStrLn, appendFile)
+import Hledger.Cli.Options
+import Hledger.Cli.Register (postingsReportAsText)
+import Hledger.Cli.Utils
+
 
 {- | Information used as the basis for suggested account names, amounts,
      etc in add prompt
@@ -43,8 +47,8 @@ data PostingState = PostingState {
 -- | Read transactions from the terminal, prompting for each field,
 -- and append them to the journal file. If the journal came from stdin, this
 -- command has no effect.
-add :: [Opt] -> [String] -> Journal -> IO ()
-add opts args j
+add :: CliOpts -> Journal -> IO ()
+add opts j
     | f == "-" = return ()
     | otherwise = do
   hPutStrLn stderr $
@@ -52,7 +56,7 @@ add opts args j
     ++"To complete a transaction, enter . when prompted for an account.\n"
     ++"To quit, press control-d or control-c."
   today <- getCurrentDay
-  getAndAddTransactions j opts args today
+  getAndAddTransactions j opts today
         `catch` (\e -> unless (isEOFError e) $ ioError e)
       where f = journalFilePath j
 
@@ -60,29 +64,29 @@ add opts args j
 -- validating, displaying and appending them to the journal file, until
 -- end of input (then raise an EOF exception). Any command-line arguments
 -- are used as the first transaction's description.
-getAndAddTransactions :: Journal -> [Opt] -> [String] -> Day -> IO ()
-getAndAddTransactions j opts args defaultDate = do
-  (t, d) <- getTransaction j opts args defaultDate
+getAndAddTransactions :: Journal -> CliOpts -> Day -> IO ()
+getAndAddTransactions j opts defaultDate = do
+  (t, d) <- getTransaction j opts defaultDate
   j <- journalAddTransaction j opts t
-  getAndAddTransactions j opts args d
+  getAndAddTransactions j opts d
 
 -- | Read a transaction from the command line, with history-aware prompting.
-getTransaction :: Journal -> [Opt] -> [String] -> Day
+getTransaction :: Journal -> CliOpts -> Day
                     -> IO (Transaction,Day)
-getTransaction j opts args defaultDate = do
+getTransaction j opts defaultDate = do
   today <- getCurrentDay
   datestr <- runInteractionDefault $ askFor "date" 
             (Just $ showDate defaultDate)
             (Just $ \s -> null s || 
              isRight (parse (smartdate >> many spacenonewline >> eof) "" $ lowercase s))
   description <- runInteractionDefault $ askFor "description" (Just "") Nothing
-  let historymatches = transactionsSimilarTo j args description
+  let historymatches = transactionsSimilarTo j (patterns_ $ reportopts_ opts) description
       bestmatch | null historymatches = Nothing
                 | otherwise = Just $ snd $ head historymatches
       bestmatchpostings = maybe Nothing (Just . tpostings) bestmatch
       date = fixSmartDate today $ fromparse $ (parse smartdate "" . lowercase) datestr
       accept x = x == "." || (not . null) x &&
-        if NoNewAccts `elem` opts
+        if no_new_accounts_ opts
             then isJust $ Foldable.find (== x) ant
             else True
         where (ant,_,_,_) = groupPostings $ journalPostings j
@@ -130,7 +134,7 @@ getPostings st enteredps = do
                 -- force a decimal point in the output in case there's a
                 -- digit group separator that would be mistaken for one
                 historicalamountstr = showMixedAmountWithPrecision maxprecisionwithpoint $ pamount $ fromJust bestmatch'
-                balancingamountstr  = showMixedAmountWithPrecision maxprecisionwithpoint $ negate $ sumMixedAmountsPreservingHighestPrecision $ map pamount enteredrealps
+                balancingamountstr  = showMixedAmountWithPrecision maxprecisionwithpoint $ negate $ sum $ map pamount enteredrealps
       amountstr <- runInteractionDefault $ askFor (printf "amount  %d" n) defaultamountstr validateamount
       let amount  = fromparse $ runParser (someamount <|> return missingamt) ctx     "" amountstr
           amount' = fromparse $ runParser (someamount <|> return missingamt) nullctx "" amountstr
@@ -184,11 +188,11 @@ askFor prompt def validator = do
 
 -- | Append this transaction to the journal's file, and to the journal's
 -- transaction list.
-journalAddTransaction :: Journal -> [Opt] -> Transaction -> IO Journal
+journalAddTransaction :: Journal -> CliOpts -> Transaction -> IO Journal
 journalAddTransaction j@Journal{jtxns=ts} opts t = do
   let f = journalFilePath j
   appendToJournalFile f $ showTransaction t
-  when (Debug `elem` opts) $ do
+  when (debug_ opts) $ do
     putStrLn $ printf "\nAdded transaction to %s:" f
     putStrLn =<< registerFromString (show t)
   return j{jtxns=ts++[t]}
@@ -200,6 +204,9 @@ appendToJournalFile f s =
     then putStr $ sep ++ s
     else appendFile f $ sep++s
     where 
+      -- appendFile means we don't need file locking to be
+      -- multi-user-safe, but also that we can't figure out the minimal
+      -- number of newlines needed as separator
       sep = "\n\n"
       -- sep | null $ strip t = ""
       --     | otherwise = replicate (2 - min 2 (length lastnls)) '\n'
@@ -208,10 +215,10 @@ appendToJournalFile f s =
 -- | Convert a string of journal data into a register report.
 registerFromString :: String -> IO String
 registerFromString s = do
-  now <- getCurrentLocalTime
-  l <- readJournalWithOpts [] s
-  return $ registerReportAsText opts $ registerReport opts (optsToFilterSpec opts [] now) l
-    where opts = [Empty]
+  d <- getCurrentDay
+  j <- readJournal' s
+  return $ postingsReportAsText opts $ postingsReport opts (optsToFilterSpec opts d) j
+      where opts = defreportopts{empty_=True}
 
 -- | Return a similarity measure, from 0 to 1, for two strings.
 -- This is Simon White's letter pairs algorithm from
