@@ -10,21 +10,26 @@ module Hledger.Cli.Version (
                            ,binaryfilename
 )
 where
+import Data.List
 import System.Info (os, arch)
+import Text.Printf
 
-import Hledger.Data.Utils
+import Hledger.Utils
 
 
 -- version and PATCHLEVEL are set by the make process
 
-version       = "0.14.0"
+version :: String
+version       = "0.15.1"
 
+patchlevel :: String
 #ifdef PATCHLEVEL
-patchlevel = "." ++ show PATCHLEVEL -- must be numeric !
+patchlevel = "." ++ show (PATCHLEVEL :: Int) -- must be numeric !
 #else
 patchlevel = ""
 #endif
 
+buildversion :: String
 buildversion  = version ++ patchlevel :: String
 
 -- | Given a program name, return a human-readable version string.  For
